@@ -1,8 +1,8 @@
-#  Copyright 2021 Collate
-#  Licensed under the Apache License, Version 2.0 (the "License");
+#  Copyright 2025 Collate
+#  Licensed under the Collate Community License, Version 1.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
-#  http://www.apache.org/licenses/LICENSE-2.0
+#  https://github.com/open-metadata/OpenMetadata/blob/main/ingestion/LICENSE
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -12,6 +12,7 @@
 Hosts the singledispatch to get DBT files
 """
 import json
+import os
 import traceback
 from collections import defaultdict
 from functools import singledispatch
@@ -78,11 +79,7 @@ def _(config: DbtLocalConfig):
     try:
         blob_grouped_by_directory = defaultdict(list)
 
-        subdirectory = (
-            config.dbtManifestFilePath.rsplit("/", 1)[0]
-            if "/" in config.dbtManifestFilePath
-            else ""
-        )
+        subdirectory = os.path.dirname(config.dbtManifestFilePath)
         blob_grouped_by_directory[subdirectory] = [
             config.dbtManifestFilePath,
             config.dbtCatalogFilePath,
@@ -247,14 +244,15 @@ def get_blobs_grouped_by_dir(blobs: List[str]) -> Dict[str, List[str]]:
     """
     blob_grouped_by_directory = defaultdict(list)
     for blob in blobs:
-        subdirectory = blob.rsplit("/", 1)[0] if "/" in blob else ""
-        blob_file_name = blob.rsplit("/", 1)[1] if "/" in blob else blob
+        subdirectory = os.path.dirname(blob)
+        blob_file_name = os.path.basename(blob)
         # We'll be processing multiple run_result files from a single dir
         # Grouping them together to process them in a single go
         if (
             DBT_MANIFEST_FILE_NAME == blob_file_name.lower()
             or DBT_CATALOG_FILE_NAME == blob_file_name.lower()
             or DBT_RUN_RESULTS_FILE_NAME in blob_file_name.lower()
+            or DBT_SOURCES_FILE_NAME == blob_file_name.lower()
         ):
             blob_grouped_by_directory[subdirectory].append(blob)
     return blob_grouped_by_directory
@@ -282,7 +280,7 @@ def download_dbt_files(
             for blob in blobs:
                 if blob:
                     reader = get_reader(config_source=config, client=client)
-                    blob_file_name = blob.rsplit("/", 1)[1] if "/" in blob else blob
+                    blob_file_name = os.path.basename(blob)
                     if DBT_MANIFEST_FILE_NAME == blob_file_name.lower():
                         logger.debug(f"{DBT_MANIFEST_FILE_NAME} found in {key}")
                         dbt_manifest = reader.read(path=blob, **kwargs)
