@@ -46,3 +46,6 @@ DEFAULT_STREAM_COLUMNS = [
         "primary_key": False,
     },
 ]
+
+SNOWFLAKE_TAG_DESCRIPTION = "SNOWFLAKE TAG VALUE"
+SNOWFLAKE_CLASSIFICATION_DESCRIPTION = "SNOWFLAKE TAG NAME"
