@@ -1,8 +1,8 @@
-#  Copyright 2021 Collate
-#  Licensed under the Apache License, Version 2.0 (the "License");
+#  Copyright 2025 Collate
+#  Licensed under the Collate Community License, Version 1.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
-#  http://www.apache.org/licenses/LICENSE-2.0
+#  https://github.com/open-metadata/OpenMetadata/blob/main/ingestion/LICENSE
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -44,6 +44,7 @@ from metadata.ingestion.api.models import Either
 from metadata.ingestion.api.steps import InvalidSourceException
 from metadata.ingestion.models.pipeline_status import OMetaPipelineStatus
 from metadata.ingestion.ometa.ometa_api import OpenMetadata
+from metadata.ingestion.source.database.column_type_parser import ColumnTypeParser
 from metadata.ingestion.source.pipeline.openlineage.models import (
     EventType,
     LineageEdge,
@@ -93,6 +94,7 @@ class OpenlineageSource(PipelineServiceSource):
         """Nothing to prepare"""
 
     def close(self) -> None:
+        self.metadata.compute_percentile(Pipeline, self.today)
         self.metadata.close()
 
     @classmethod
@@ -216,9 +218,12 @@ class OpenlineageSource(PipelineServiceSource):
         try:
             fields = table_input["facets"]["schema"]["fields"]
 
-            # @todo check if this way of passing type is ok
             columns = [
-                Column(name=f.get("name"), dataType=f.get("type").upper())
+                Column(
+                    name=f.get("name"),
+                    dataTypeDisplay=f.get("type").upper(),
+                    dataType=ColumnTypeParser.get_column_type(f.get("type").upper()),
+                )
                 for f in fields
             ]
             return columns
